<?php

namespace Folha\Logic\ControlePonto;

use Folha\Entities\FpaLotelancevento;
use Folha\Entities\PtoResumositufreq;
use Folha\Entities\PtoSitufreqparam;
use Folha\Logic\ImportacaoLoteEvento\LoteEvento;
use Setforms2\Shared\Registry;

class GeracaoLoteEvento
{
    private $criterio;
    private $inicio;
    private $fim;
    private $dtCompetencia;
    private $nrTipoVinculo;
    private $nrPeriodoApuracao;
    private $dtPeriodoIni;
    private $dtPeriodoFim;
    private $nrOrgPadrao;
    private $nrOrg;

    /**
     * @param $resumos      Resumos de Situação de Frequencia existentes para geração do Lote de Eventos - Entidade: PTO_RESUMOSITUFREQ.
     * @param $resumosLote  Resumos de situação de Frequencia já agrupados no formato do Lote de Eventos.
     */
    private $resumos;
    private $resumosLote;

    public function __construct($criterio, $inicio, $fim, $dtCompetencia, $nrTipoVinculo, $nrPeriodoApuracao, $dtPeriodoIni, $dtPeriodoFim, $nrOrgPadrao, $nrOrg)
    {
        $this->criterio = $criterio;
        $this->inicio = $inicio;
        $this->fim = $fim;
        $this->dtCompetencia = $dtCompetencia;
        $this->nrTipoVinculo = $nrTipoVinculo;
        $this->nrPeriodoApuracao = $nrPeriodoApuracao;
        $this->dtPeriodoIni = $dtPeriodoIni;
        $this->dtPeriodoFim = $dtPeriodoFim;
        $this->nrOrgPadrao = $nrOrgPadrao;
        $this->nrOrg = $nrOrg;

        $this->resumos = self::getResumoSituFreq();
        $this->resumosLote = self::getResumoSituFreqLote();
    }

    /**
     * São gerados os lotes de eventos com base nos resumos de situação de frequência encontrados.
     */
    public function geraLote()
    {
        $nrorg = \FPA::getNrorgTrabalho();

        foreach ($this->resumosLote as $item) {
            $nrsitufrequencia = '';

            $vrRefEvento = (is_null($item['QTREFAPURSITUFREQ'])) ? 0 : $item['QTREFAPURSITUFREQ'];

            $itensLote[] = array(
                'NRVINCULOM' => $item['NRVINCULOM'],
                'NMVINCULOM' => '',
                'NREVENTOM' => $item['NREVENTOM'],
                'NMEVENTOH' => '',
                'VRREFEVENTO' => $vrRefEvento
            );
        }

        try {
            Registry::getEntityManager()->getConnection()->beginTransaction();

            $loteEvento = new LoteEvento($nrsitufrequencia, $nrorg, $itensLote, 'PTO');

            foreach ($this->resumos as $item) {
                $resumoSituFreq = PtoResumositufreq::findOneByNrresumositufreqAndNrorg($item['NRRESUMOSITUFREQ'], $item['NRORG']);
                $resumoSituFreq->setNrlotelancevento($loteEvento->getNrlote());
                $resumoSituFreq->persist();
            }

            Registry::getEntityManager()->flush();

            Registry::getEntityManager()->getConnection()->commit();

            return $loteEvento->getNrlote();
        } catch (\Exception $e) {
            Registry::getEntityManager()->getConnection()->rollBack();
            throw $e;
        }
    }

    /**
     * Busca os resumos de situação de frequência com base nos critérios de seleção e verifica existem dados para gerar o lote de eventos e, caso exista, verifica se todas
     * situações de frequência possuem evento cadastrado retornando uma mensagem de erro. Caso esteja ok, retorna os resumos de situação de frequência.
     *
     * @return mixed
     * @throws \Exception
     */
    public function getResumoSituFreq()
    {
        $resumos = PtoResumositufreq::getResumoPorSelecao($this->criterio, $this->inicio, $this->fim, $this->dtCompetencia, $this->nrTipoVinculo, $this->nrPeriodoApuracao, $this->dtPeriodoIni, $this->dtPeriodoFim, $this->nrOrgPadrao, $this->nrOrg, 'S');

        if(!$resumos){
            throw new \Exception(tk_get_message_by_name("msg_pto00017_sem_dados"));
        }
        else {
            $listSituFreq = GeracaoLoteEvento::validaSituFreqSemEvento($resumos);

            if($listSituFreq){
                throw new \Exception(tk_get_message_by_name("msg_pto00017_situfreq_sem_evento", array($listSituFreq)));
            }
            else {
                return $resumos;
            }
        }
    }

    /**
     * Com base nos critérios de seleção retorna os valores dos resumos de situação de frequência agrupados por evento e vinculo, já no formato necessário para gerar os itens
     * do lote de eventos da folha.
     * 
     * @return mixed
     * @throws \Exception
     */
    public function getResumoSituFreqLote()
    {
        $resumosLote = PtoResumositufreq::getResumoParaLote($this->criterio, $this->inicio, $this->fim, $this->dtCompetencia, $this->nrTipoVinculo, $this->nrPeriodoApuracao, $this->dtPeriodoIni, $this->dtPeriodoFim, $this->nrOrgPadrao, $this->nrOrg);

        if(!$resumosLote){
            throw new \Exception(tk_get_message_by_name("msg_pto00017_erros_geracao"));
        }
        else {
            return $resumosLote;
        }
    }

    public function validaSituFreqSemEvento($resumos)
    {
        $listSituFreq = '';

        foreach($resumos as $item){
            $where = "NRSITUFREQUENCIA = :P_NRSITUFREQUENCIA AND (NRORG = :P_NRORG OR NRORG = 0)";
            $params = array(
                'P_NRORG' => $item['NRORG'],
                'P_NRSITUFREQUENCIA' => $item['NRSITUFREQUENCIA']);
            $situfreqparam = PtoSitufreqparam::getObjectByWhere($where, $params);

            if(!$situfreqparam || !$situfreqparam->getNreventom()){
                if(strpos($listSituFreq, $item['NRSITUFREQUENCIA']) === false){
                    $listSituFreq .= ($listSituFreq == '') ? $item['NRSITUFREQUENCIA'] : ", ".$item['NRSITUFREQUENCIA'];
                }
            }
        }

        return ($listSituFreq == '') ? false : $listSituFreq;
    }

    /**
     * É gerado um array para ser retornado como datasource da tab de simulação da geração do lote de eventos.
     *
     * @param $resumos      Array dos resumos de situação de frequencia - Entidade: PTO_RESUMOSITUFREQ
     * @return array        É retornado um array contendo as chaves com os nomes dos campos da tab de visualização
     */
    public function geraVisualizacaoGrid()
    {
        foreach ($this->resumosLote as $item) {

            /*Só são exibidos os itens que possuem valor pois só estes são gerados no lote*/
            if ($item['QTREFAPURSITUFREQ']) {

                if ($item['IDMEDIAVALREF'] == 'REFERENCIA') {
                    $vrreffolhalote = (is_null($item['QTREFAPURSITUFREQ'])) ? 0 : $item['QTREFAPURSITUFREQ'];
                    $vreventolote = '';
                } else {
                    $vrreffolhalote = '';
                    $vreventolote = (is_null($item['QTREFAPURSITUFREQ'])) ? 0 : $item['QTREFAPURSITUFREQ'];
                }

                /*Array preenchido com o os nomes dos campos da tab.*/
                $result[] = array(
                    'NRVINCULOM' => $item['NRVINCULOM'],
                    'NMVINCULOM' => $item['NMVINCULOM'],
                    'NREVENTOM' => $item['NREVENTOM'],
                    'NMEVENTOH' => $item['NMEVENTOH'],
                    'VRREFERENCIA' => $vrreffolhalote,
                    'VREVENTO' => $vreventolote
                );
            }
        }

        return $result;
    }

    public function abreWindowLote($nrlotelancevento)
    {
        $lote = FpaLotelancevento::findOneByNrlotelancevento($nrlotelancevento);

        $filtro = array(
            'NRLOTELANCEVENTO' => $lote->getNrlotelancevento(),
            'DSLOTELANCEVENTO' => $lote->getDslotelancevento(),
            'DTLOTELANCEVENTO' => $lote->getDtlotelancevento(),
            'DTCOMPETLOTEEVE' => $lote->getDtcompetloteeve(),
            'IDSTATUSLOTELANC' => $lote->getIdstatuslotelanc()
        );

        tk_open_window('hrm_lote_evento', false, $filtro);
    }
} 