<?php
namespace Teknisa\Entities;

use Setforms2\Shared\Registry;

class Remprodnutr extends Base\Remprodnutr
{

    /**
     * Insere uma nova remessa no banco.
     *
     * @param String $nrorg       Nrorg
     * @param String $dataAtual   DataAtual
     * @param String $cdoperador  CdOperador
     * @param String $idtpremessa Tipo Remessa
     *
     * @return mixed.
     * @throws \Exception
     */
    public static function novaRemessa(
        $nrorg, $dataAtual, $cdoperador, $idtpremessa
    ) {
        $novoCodigo = self::novoNrremessa();
        //
        $remessa = new self();
        $remessa->setNrremessa($novoCodigo);
        $remessa->setIdsitremessa('N');
        $remessa->setIdtpremessa($idtpremessa);
        $remessa->setDtenvio($dataAtual);
        $remessa->setDtinclusao($dataAtual);
        $remessa->setDtultatu($dataAtual);
        $remessa->setNrorg($nrorg);
        $remessa->setNrorginclusao($nrorg);
        $remessa->setNrorgultatu($nrorg);
        $remessa->setCdoperinclusao($cdoperador);
        $remessa->setCdoperultatu($cdoperador);
        $remessa->setIdativo('S');
        $remessa->persist();
        //
        return $novoCodigo;
    }

    /**
     * Gera novo código para remessa. Considera apenas os registros existentes no
     * banco pois não é possível gerar mais de uma remessa por vez.
     *
     * @return mixed.
     */
    public static function novoNrremessa()
    {
        $remessa = self::getObjectByWhere(
            "1 = 1",
            array(),
            "NRREMESSA DESC"
        );
        if ($remessa instanceof self) {
            return ($remessa->getNrremessa() + 1);
        } else {
            return 1;
        }
    }

    /**
     * Atualiza a Situação da Remessa recebida por parâmetro.
     *
     * @param Remprodnutr $remessa      Remessa
     * @param String      $idsitremessa Situação
     *
     * @return null
     */
    public static function atualizaSituacaoRemessa ($remessa, $idsitremessa)
    {
        if ($remessa instanceof self) {
            $remessa->setIdsitremessa($idsitremessa);
            $remessa->persist();
        }
    }

    /**
     * Atualiza a Mensagem de Erro da Remessa recebida por parâmetro.
     *
     * @param Remprodnutr $remessa  Remessa
     * @param String      $menserro Mensagem de Erro
     *
     * @return null
     */
    public static function atualizaMensErroRemessa ($remessa, $menserro)
    {
        if ($remessa instanceof self) {
            $remessa->setDsmenserro($menserro);
            $remessa->persist();
        }
    }

    /**
     * Exclui a remessa que contém as PKs passadas por parâmetro.
     *
     * @param String $nrorg       Organização
     * @param String $nrremessa   Número Remessa
     * @param String $idtpremessa Tipo Remessa
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiRemessa ($nrorg, $nrremessa, $idtpremessa)
    {
        $reg = self::getObjectByWhere(
            "NRORG = ? AND NRREMESSA = ? AND IDTPREMESSA = ?",
            array($nrorg, $nrremessa, $idtpremessa)
        );

        if ($reg instanceof self) {
            $conn = Registry::getEntityManager();
            try {
                $conn->beginTransaction();
                $reg ->delete();
                $conn->flush();
                $conn->commit();
            } catch (\Exception $e) {
                $conn->rollback();
                throw $e;
            }
        }
    }

}