/*

Descrio: Criada para encaixar a data de produo no horario do turno de trabalho.
Criao: Welbert G. Moitinha
Data: 28/06/2010

*/


create or replace
FUNCTION ENCAIXA_PRODUCAO_TURNO (
    P_CDFILIAL         IN  FILIAL.CDFILIAL%TYPE, 
    P_DTAPRODUZIR      IN  DATE,       
    P_CDPERIPRODU      IN  CALEPRODUCAO.CDPERIPRODU%TYPE DEFAULT '',
    P_QTDEMINUTOSUTIL  IN  FLOAT DEFAULT 0,
    P_MSGERROR         OUT VARCHAR,
    P_IDUTEQUIPLAPROD IN VARCHAR DEFAULT 'N')RETURN DATE 
IS 

/************************************************
           DECLARAO DE VARIVEIS
  *************************************************/
   
   V_VEXISHORATUR   TURNO_PRODUCAO.HRINITURNO%TYPE;
   V_NRTURNO        TURNO.NRTURNO%TYPE;   
   V_DTAPRODUZIR    DATE;   
   V_TURNOFIMANT    TURNO_PRODUCAO.HRFIMTURNO%TYPE;
   V_VERIFTURNO     TURNO_PRODUCAO.HRFIMTURNO%TYPE;  
   V_DTPRODUCAO     DATE;   
   V_DTPRODFIN      DATE;   
   V_DATA_CALE      DATE;   
   V_DTAUXI         DATE;   
   V_QTHRTURNO      FLOAT;
   V_NRTURNOALT     TURNO_PRODUCAO.NRTURNO%TYPE; 
   V_DTHRINICTURNO  DATE;
   V_DTHRFIMTURNO   DATE;
   V_QTHRRESTURNO   FLOAT;
   
   
 --  V_VEXISHORATUR   TURNO_PRODUCAO.HRINITURNO%TYPE; 
     V_TURNOINIANT    TURNO_PRODUCAO.HRINITURNO%TYPE; 
 --  V_TURNOFIMANT    TURNO_PRODUCAO.HRFIMTURNO%TYPE; 
   V_TURNOAUXI      INTEGER; 
   V_DTTURNOAUXI    NUMBER; 
 --  V_DTAPRODUZIR DATE; 
   
  /************************************************
           DECLARAO DE CURSORES
  *************************************************/
   
  --Cursor para buscar a maior data no calendrio da produo,
  --que seja menor ou igual  data armazenada em V_DATA_CALE
  CURSOR C_CALEN_PROD IS
  SELECT
      MAX(DTPRODUCAO) AS DTPRODUCAO     
  FROM
     CALEPRODUCAO
  WHERE
     (CDFILIAL             = P_CDFILIAL)     AND
     (CDPERIPRODU          = P_CDPERIPRODU)  AND
     (TO_DATE(DTPRODUCAO) <= TO_DATE(V_DATA_CALE));    
  
   
BEGIN 


IF P_IDUTEQUIPLAPROD = 'N' THEN
   V_DTAPRODUZIR := P_DTAPRODUZIR; 
 
   SELECT 
      MAX(T.HRINITURNO) 
   INTO 
      V_VEXISHORATUR 
   FROM 
      (SELECT 
          NRTURNO, 
          HRINITURNO, 
          HRFIMTURNO, 
          DECODE(HRFIMTURNO - HRINITURNO, ABS(HRFIMTURNO - HRINITURNO), 
                 HRFIMTURNO, TO_NUMBER(HRFIMTURNO) + 2400) AS HRFINPRODU 
       FROM TURNO_PRODUCAO 
       WHERE 
          (CDFILIAL = P_CDFILIAL) 
      ) T 
   WHERE 
      (DECODE(T.HRFIMTURNO - T.HRINITURNO, ABS(T.HRFIMTURNO - T.HRINITURNO), 
              TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI')), 
              DECODE(T.HRINITURNO - TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI')), ABS(T.HRINITURNO - TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI'))), 
              TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI')) + 2400, T.HRINITURNO)) BETWEEN T.HRINITURNO AND T.HRFINPRODU); 
    
   IF V_VEXISHORATUR IS NULL THEN 
      SELECT 
         MAX(T.HRINITURNO) 
      INTO 
         V_TURNOINIANT 
      FROM 
         (SELECT 
             MAX(HRINITURNO) AS HRINITURNO 
          FROM 
             TURNO_PRODUCAO 
          WHERE 
             (CDFILIAL   = P_CDFILIAL)                                  AND 
             (HRINITURNO > TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI'))) 
         ) A, 
         TURNO_PRODUCAO T 
      WHERE 
         (T.CDFILIAL   = P_CDFILIAL)   AND 
         (T.HRINITURNO = A.HRINITURNO); 
      V_DTTURNOAUXI := TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || V_TURNOINIANT, 'DD/MM/YYYY HH24MI') - V_DTAPRODUZIR; 
      SELECT 
         MAX(T.HRFIMTURNO) 
      INTO 
         V_TURNOFIMANT 
      FROM 
         (SELECT 
             MAX(HRFIMTURNO) AS HRFIMTURNO 
          FROM 
             TURNO_PRODUCAO 
          WHERE 
             (CDFILIAL   = P_CDFILIAL)                                  AND 
             (HRFIMTURNO < TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI'))) 
         ) A, 
         TURNO_PRODUCAO T 
      WHERE 
         (T.CDFILIAL   = P_CDFILIAL)   AND 
         (T.HRFIMTURNO = A.HRFIMTURNO); 
      V_TURNOAUXI := TO_CHAR(TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || V_TURNOFIMANT, 'DD/MM/YYYY HH24MI') - V_DTTURNOAUXI, 'HH24MI'); 
      IF NOT V_TURNOINIANT IS NULL THEN 
         V_DTAPRODUZIR := TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || LPAD(TO_CHAR(V_TURNOAUXI),4,'0'), 'DD/MM/YYYY HH24MI'); 
      END IF; 
   END IF; 
   RETURN(V_DTAPRODUZIR); 

ELSE

  V_DTAPRODUZIR := P_DTAPRODUZIR;
  P_MSGERROR    := '';
  
  IF V_DTAPRODUZIR < SYSDATE THEN  
    RETURN NULL; 
  END IF;
  
  
  /**************************************************************************************************
    0. Verifica se existe algum turno com quantidade de horas suficiente (Independente da Data)
  **************************************************************************************************/
  BEGIN
      SELECT
         MAX(NRTURNO)
      INTO
         V_VERIFTURNO
      FROM 
         TURNO_PRODUCAO
      WHERE
         (CDFILIAL = P_CDFILIAL) AND
         (           
            --Nos Turnos que iniciam e terminam no mesmo dia               
           (((TO_DATE(TO_CHAR(TRUNC(SYSDATE),'DD/MM/YYYY')||HRFIMTURNO,'DD/MM/YYYY HH24MI')-
             TO_DATE(TO_CHAR(TRUNC(SYSDATE),'DD/MM/YYYY')||HRINITURNO,'DD/MM/YYYY HH24MI'))>=P_QTDEMINUTOSUTIL
            )AND HRFIMTURNO > HRINITURNO  )              
           OR              
           --Nos Turnos que iniciam e terminam em dias diferentes
           (((TO_DATE(TO_CHAR(TRUNC(SYSDATE+1),'DD/MM/YYYY')||HRFIMTURNO,'DD/MM/YYYY HH24MI')-
              TO_DATE(TO_CHAR(TRUNC(SYSDATE),'DD/MM/YYYY')||HRINITURNO,'DD/MM/YYYY HH24MI'))>=P_QTDEMINUTOSUTIL
            )AND HRFIMTURNO < HRINITURNO ) 
         );
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_VERIFTURNO := NULL;
   END;
          
   /**************************************************************************************
          0.I) Se no existir, retorna erro("No existe turno que comporte a etapa")
   ***************************************************************************************/
   IF V_VERIFTURNO IS NULL THEN
      --No existe turno que comporte a etapa
      P_MSGERROR := 'Turn_N_Exi';
      RETURN NULL;
   END IF;
    
  /************************************************
     1. verificar o turno em que esta a hora
  *************************************************/
      
   --BUSCA O TURNO EM QUE SE ENCONTRA A DATA A PRODUZIR
   BEGIN
      SELECT
         HRINITURNO,
         NRTURNO         
      INTO
         V_VEXISHORATUR, V_NRTURNO
      FROM TURNO_PRODUCAO
      WHERE
      (CDFILIAL = P_CDFILIAL) AND
      (           
            --Nos Turnos que iniciam e terminam no mesmo dia, a hora verificada deve estar entre a hora inicial e a final
            (( TO_NUMBER(TO_CHAR(V_DTAPRODUZIR,'HH24MI'))  BETWEEN
                TO_NUMBER(HRINITURNO) AND TO_NUMBER(HRFIMTURNO)
             ) AND HRFIMTURNO > HRINITURNO  ) 
             
            OR 
             
            --Nos Turnos que iniciam e terminam em dias diferentes, a hora verificada deve ser maior que a inicial OU menor que a final
            ((
               (TO_NUMBER(TO_CHAR(V_DTAPRODUZIR,'HH24MI')) >= TO_NUMBER(HRINITURNO)) OR
               (TO_NUMBER(TO_CHAR(V_DTAPRODUZIR,'HH24MI')) <= TO_NUMBER(HRFIMTURNO))
            )AND HRFIMTURNO < HRINITURNO ) 
      );
   EXCEPTION
       WHEN NO_DATA_FOUND THEN
          V_VEXISHORATUR := NULL; 
   END;
   
   /*************************************************
      2. Se no estiver em nenhum turno, faa: 
   *************************************************/
    
   /* se no possuir, verifica o turno anterior    <== MUDAR PARA JA BUSCAR O TURNO ANTERIOR, SE EXISTIR, */  
   IF V_VEXISHORATUR IS NULL THEN
     BEGIN
     
          /*****************************************************************************************
                  2.a) Busca um turno anterior (No mesmo dia), com qt. horas suficiente   <== ALTERAR
          ******************************************************************************************/
          
          SELECT
             MAX(T.HRFIMTURNO), MAX(NRTURNO)
          INTO
             V_TURNOFIMANT, V_NRTURNO
          FROM
             (SELECT
                 MAX(HRFIMTURNO) AS HRFIMTURNO
              FROM
                 TURNO_PRODUCAO
              WHERE
                 (CDFILIAL   = P_CDFILIAL)                                  AND
                 (HRFIMTURNO < TO_NUMBER(TO_CHAR(V_DTAPRODUZIR, 'HH24MI')))
             ) A,
             TURNO_PRODUCAO T
          WHERE
             (T.CDFILIAL   = P_CDFILIAL)   AND
             (T.HRFIMTURNO = A.HRFIMTURNO)
          GROUP BY
             NRTURNO;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
           V_TURNOFIMANT := NULL; 
      END;
      
      /**************************************************************************************************
          2.b) Se houver, Data_Produzir = (trunc(Data_Produzir) + hrfimturno) - qtHorasNecessarias        
      **************************************************************************************************/
      IF NOT V_TURNOFIMANT IS NULL THEN
         V_DTAPRODUZIR := TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || V_TURNOFIMANT, 'DD/MM/YYYY HH24MI') - P_QTDEMINUTOSUTIL;
     
      /*************************************************
              2.c) Se no houver, faa:    
      *************************************************/
      ELSE            
             /*************************************************************************************************************
               2.c.I)  Revalida, recursivamente, no dia anterior, a partir das 23:59 menos a Qtde. Horas Necessarias:
             **************************************************************************************************************/ 
             V_DTAPRODUZIR := ENCAIXA_PRODUCAO_TURNO(P_CDFILIAL,(TRUNC(V_DTAPRODUZIR)-(1/24/60)-P_QTDEMINUTOSUTIL),
                                                          P_CDPERIPRODU,P_QTDEMINUTOSUTIL,P_MSGERROR,'S');
             RETURN V_DTAPRODUZIR;          
      END IF;
   END IF;

  /**************************************************************************************************
    3.  Busca a maior data do calendario da producao, que seja menor ou igual a Data_Produzir
  **************************************************************************************************/
  
  -----------------------------------------------------------------------------------------------
  V_DATA_CALE := V_DTAPRODUZIR; --O cursor C_CALEN_PROD busca a maior data no calendario da producao que seja <= V_DATA_CALE
  OPEN C_CALEN_PROD;
  FETCH C_CALEN_PROD INTO V_DTPRODUCAO; --Salva a data em V_DTPRODUCAO
  CLOSE C_CALEN_PROD;
  -----------------------------------------------------------------------------------------------
  
  
  /**************************************************************************************************    
      4.  Se no encontrar data, retorna erro("Calendario no parametrizado")  
  **************************************************************************************************/
  IF V_DTPRODUCAO IS NULL THEN  
      P_MSGERROR := 'Cale_N_param';
      RETURN NULL;  
  
  /******************************
     5.  Se encontrar, faa:
  ******************************/
  ELSE      
      /*************************************************************************
         5.a) comparar a data retornada com Data_Produzir. Se for MENOR, faa:
      **************************************************************************/
      IF TRUNC(V_DTPRODUCAO) < TRUNC(V_DTAPRODUZIR) THEN
         
         /******************************************************************************************************************
           5.a.I)  Revalida, recursivamente, na maior data de produo encontrada, a partir das 23:59 menos a Qtde. Horas 	 
         *******************************************************************************************************************/
         V_DTAPRODUZIR := ENCAIXA_PRODUCAO_TURNO(P_CDFILIAL,(TRUNC(V_DTPRODUCAO+1)-(1/24/60)-P_QTDEMINUTOSUTIL),P_CDPERIPRODU,
                                                      P_QTDEMINUTOSUTIL,P_MSGERROR,'S');
         RETURN V_DTAPRODUZIR;
      END IF;      
  END IF;
  
  
  /******************************************************************************************************************
   6. Busca a data/hora de incio e trmino do turno, verificando se o mesmo possui quantidade de horas suficientes.
  ******************************************************************************************************************/
  
   /* VERIFICA SE A DATA FINAL ESTA DENTRO DO TURNO*/
   /* H - HOJE   - Data a Produzir
      A - AMANHA - Dia seguinte  Data a Produzir
      O - ONTEM  - Dia seguinte  Data a Produzir*/
   BEGIN
      SELECT INICIO, FIM,V_DTAPRODUZIR, QTHRTURNO
      INTO V_DTHRINICTURNO,V_DTHRFIMTURNO, V_DTAUXI, V_QTHRTURNO
      FROM
      (
      
      -- Se o horrio de Trmino do Turno for maior que a de incio, o turno est todo em um dia
      SELECT T.HRINITURNO, T.HRFIMTURNO,
              (FIM - INICIO)AS QTHRTURNO,
               INICIO, FIM, 'H' AS IDDIA

      FROM TURNO_PRODUCAO T,
           (SELECT
               TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRINITURNO, 'DD/MM/YYYY HH24MI') AS INICIO,
               TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRFIMTURNO, 'DD/MM/YYYY HH24MI') AS FIM,
               CDFILIAL, NRTURNO
             FROM
                TURNO_PRODUCAO
             WHERE
                NRTURNO = V_NRTURNO AND
                CDFILIAL = P_CDFILIAL )X
      WHERE
          T.CDFILIAL = P_CDFILIAL AND
          T.CDFILIAL = X.CDFILIAL  AND
          T.NRTURNO = X.NRTURNO    AND
          --Indica que o turno comea e termina no mesmo dia
          FIM > INICIO 
                    
      UNION
       
      /*  Se o horrio de Incio for maior que o de trmino, indica que o turno inicia e termina em dias diferentes.
             Ocorrem, ento, duas possibilidades (a e b):                               */
             
      /*  a) Se a hora de incio da produo for MENOR que a hora de trmino do turno, significa que
               o incio da produo ocorre no mesmo dia que o trmino do turno, logo:
               
                  Incio  do Turno: Data Anterior
                  Trmino do Turno: Data de Incio da produo (V_DTAPRODUZIR)   
      */
      SELECT T.HRINITURNO, T.HRFIMTURNO,
              ((FIM) - (INICIO-1)) AS QTHRTURNO,
               INICIO-1, FIM, 'O' AS IDDIA
       FROM TURNO_PRODUCAO T,
           (SELECT
               TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRINITURNO, 'DD/MM/YYYY HH24MI') AS INICIO,
               TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRFIMTURNO, 'DD/MM/YYYY HH24MI') AS FIM,
               CDFILIAL, NRTURNO
             FROM
                TURNO_PRODUCAO
             WHERE
                NRTURNO = V_NRTURNO AND
                CDFILIAL = P_CDFILIAL)X
      WHERE
          T.CDFILIAL = P_CDFILIAL AND
          T.CDFILIAL = X.CDFILIAL  AND
          T.NRTURNO = X.NRTURNO    AND
          --Indica que o Turno Inicia e Termina em dias diferentes
          INICIO > FIM  AND
          --Indica que o turno comea no dia anterior             
          (
             V_DTAPRODUZIR <
             TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRFIMTURNO, 'DD/MM/YYYY HH24MI') 
          )
          
      UNION
          
       /* b) Se a hora de incio da produo for MAIOR que a hora de trmino do turno, significa que
               o incio da produo ocorre no dia anterior ao trmino do turno, logo:
               
                  Incio  do Turno: Data de Incio da produo (V_DTAPRODUZIR) 
                  Trmino do Turno: Data Anterior (V_DTAPRODUZIR - 1)  
       */
       SELECT T.HRINITURNO, T.HRFIMTURNO,
              ((FIM+1) - (INICIO)) AS QTHRTURNO,
               INICIO, FIM+1, 'A' AS IDDIA
       FROM TURNO_PRODUCAO T,
           (SELECT
               TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRINITURNO, 'DD/MM/YYYY HH24MI') AS INICIO,
               TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRFIMTURNO, 'DD/MM/YYYY HH24MI') AS FIM,
               CDFILIAL, NRTURNO
             FROM
                TURNO_PRODUCAO
             WHERE
                NRTURNO = V_NRTURNO AND
                CDFILIAL = P_CDFILIAL)X
       WHERE
          T.CDFILIAL = P_CDFILIAL AND
          T.CDFILIAL = X.CDFILIAL  AND
          T.NRTURNO = X.NRTURNO    AND
          --Indica que o Turno Inicia e Termina em dias diferentes
          INICIO > FIM  AND
          --Indica que o turno comea no dia da produo e termina no dia seguinte               
          (
             V_DTAPRODUZIR >
             TO_DATE(TO_CHAR(TRUNC(V_DTAPRODUZIR), 'DD/MM/YYYY') || HRFIMTURNO, 'DD/MM/YYYY HH24MI') 
          )
        )
        WHERE
        (QTHRTURNO >= P_QTDEMINUTOSUTIL);     
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_DTAUXI := NULL;
      END;
      
  /*********************************************  
         7. Se no encontrar, faa: 
  **********************************************/
  IF V_DTAUXI IS NULL THEN

       /************************************************************************************************************
           7.1) Revalida, recursivamente, no dia anterior, a partir das 23:59 menos a Qtde. Horas Necessarias
       *************************************************************************************************************/
        V_DTAPRODUZIR := ENCAIXA_PRODUCAO_TURNO(P_CDFILIAL,(TRUNC(V_DTAPRODUZIR)-(1/24/60)-P_QTDEMINUTOSUTIL),P_CDPERIPRODU,
                                                     P_QTDEMINUTOSUTIL,P_MSGERROR,'S');
        RETURN V_DTAPRODUZIR;
  
  /***************************  
    8. Se encontrar, faa:     
  ****************************/
  ELSE     
       
      /**************************************************************************************************************** 
         8.a).  Verifica se a data final est dentro do turno (Se tiver executado os passos 2.c.II.III, 5.a.III, 
                ou 7.a, sempre estar, mas verifica mesmo assim, por segurana). Se estiver fora, mas houver     
                quantidade de horas suficiente no turno, faa:                                                  
      *****************************************************************************************************************/
       
       
   /* SE A HORA DE TRMINO DO TURNO FOR MENOR QUE A DATA DE ENTREGA MAS HOUVER TEMPO SUFICIENTE (QUANTIDADE DE HORAS)
      PARA REALIZAR A PRODUO NO TURNO, AS DATAS DE INCIO E TRMINO DA PRODUO SO ADIANTADAS PARA ENCAIXAR NO TURNO*/   
      IF V_DTHRFIMTURNO < (V_DTAPRODUZIR + P_QTDEMINUTOSUTIL) THEN 
      
        /***********************************************************************
          8.a.I) Data_Produzir = data/hora final do turno - qtHorasNecessarias
        ************************************************************************/
         V_DTAPRODUZIR := V_DTHRFIMTURNO - P_QTDEMINUTOSUTIL; 
      END IF;
  END IF;
  
  /******************************************************************************************************
        9) Comparar as datas inicial e final do turno com a data do calendrio,(Igual ao passo 3). 
           Se alguma das duas for maior que a mesma, faa:
  ******************************************************************************************************/
  
  
  ----------------------------------------------------------------------------------------------
  --Busca Maior Data no calendrio da produo, a partir da data de incio da produo
  --O cursor C_CALEN_PROD busca a maior data no calendario da producao que seja <= V_DATA_CALE
  V_DATA_CALE := V_DTAPRODUZIR;         
  OPEN C_CALEN_PROD;
  --Salva a data em V_DTPRODUCAO
  FETCH C_CALEN_PROD INTO V_DTPRODUCAO; 
  CLOSE C_CALEN_PROD;  
  -----------------------------------------------------------------------------------------------
  --Busca Maior Data no calendrio da produo, a partir da data de trmino da produo
  
  --O cursor C_CALEN_PROD busca a maior data no calendario da producao que seja <= V_DATA_CALE
  V_DATA_CALE := V_DTAPRODUZIR+P_QTDEMINUTOSUTIL; 
  OPEN C_CALEN_PROD;
  --Salva a data em V_DTPRODFIN
  FETCH C_CALEN_PROD INTO V_DTPRODFIN;            
  CLOSE C_CALEN_PROD;
  -----------------------------------------------------------------------------------------------
  
  /***************************************************************************************************************************
    9.a) Revalida, recursivamente, na maior data de produo encontrada, a partir das 23:59 menos a Qtde. Horas Necessrias
  ****************************************************************************************************************************/
  IF (TRUNC(V_DTPRODUCAO) < TRUNC(V_DTAPRODUZIR)) THEN 
     V_DTAPRODUZIR := ENCAIXA_PRODUCAO_TURNO(P_CDFILIAL,(TRUNC(V_DTPRODUCAO+1)-(1/24/60)-P_QTDEMINUTOSUTIL),P_CDPERIPRODU,
                                                  P_QTDEMINUTOSUTIL,P_MSGERROR,'S');	
  END IF;
  IF (TRUNC(V_DTPRODFIN) < TRUNC(V_DTAPRODUZIR+P_QTDEMINUTOSUTIL)) THEN 
     V_DTAPRODUZIR := ENCAIXA_PRODUCAO_TURNO(P_CDFILIAL,(TRUNC(V_DTPRODFIN+1)-(1/24/60)-P_QTDEMINUTOSUTIL),P_CDPERIPRODU,
                                                  P_QTDEMINUTOSUTIL,P_MSGERROR,'S');	
  END IF;
  
  /*****************************************
        10. Retornar Data_Produzir
  *****************************************/
  RETURN(V_DTAPRODUZIR);
END IF;   
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ENCAIXA_PRODUCAO_TURNO','5.05.016'); END;
/
